package game

import (
	"log"
	"os"

	"code.rocketnine.space/tslocum/boxbrawl/entity"
	"code.rocketnine.space/tslocum/boxbrawl/system"
	"code.rocketnine.space/tslocum/boxbrawl/world"
	"code.rocketnine.space/tslocum/etk"
	"code.rocketnine.space/tslocum/gohan"
	"github.com/hajimehoshi/ebiten/v2"
)

type Game struct{}

func NewGame() (*Game, error) {
	g := &Game{}

	entity.NewOnceEntity()

	gohan.AddSystem(&system.UISystem{})

	return g, nil
}

func (g *Game) Layout(outsideWidth, outsideHeight int) (screenWidth, screenHeight int) {
	if outsideWidth != world.ScreenWidth || outsideHeight != world.ScreenHeight {
		if world.ScreenWidth != 0 || world.ScreenHeight != 0 {
			etk.Layout(outsideWidth, outsideHeight)
		}
		world.ScreenWidth, world.ScreenHeight = outsideWidth, outsideHeight
	}
	return world.ScreenWidth, world.ScreenHeight
}

func (g *Game) Update() error {
	if ebiten.IsWindowBeingClosed() {
		g.Exit()
		return nil
	}

	return gohan.Update()
}

func (g *Game) Draw(screen *ebiten.Image) {
	err := gohan.Draw(screen)
	if err != nil {
		log.Fatal(err)
	}
}

func (g *Game) Exit() {
	os.Exit(0)
}
