package system

import (
	"fmt"
	"log"

	"code.rocketnine.space/tslocum/boxbrawl/component"
	"code.rocketnine.space/tslocum/boxbrawl/world"
	"code.rocketnine.space/tslocum/etk"
	"code.rocketnine.space/tslocum/gohan"
	"github.com/hajimehoshi/ebiten/v2"
	"github.com/hajimehoshi/ebiten/v2/ebitenutil"
	"github.com/hajimehoshi/ebiten/v2/inpututil"
)

const uiStartPrompt = `Box Brawl
`
const uiComputerPrompt = `Press <Enter> to play against the computer.`
const uiHostPrompt = `Press <H> to host a match against a remote player.`
const uiRemotePrompt = `Type an IP address and port (address:port) to play against a remote player.`
const uiConnectPrompt = `Press <Enter> to connect.`

type UISystem struct {
	*component.Once

	initialized bool
	buffer      *etk.Text
	debugImg    *ebiten.Image
}

func (u *UISystem) initialize() {
	u.buffer = etk.NewText("")
	u.updateBuffer()

	inputDemo := etk.NewFlex()
	inputDemo.SetVertical(true)

	inputDemo.AddChild(u.buffer)

	etk.SetRoot(inputDemo)

	etk.Layout(world.ScreenWidth, world.ScreenHeight)

	u.debugImg = ebiten.NewImage(128, 128)

	u.initialized = true
}

func (u *UISystem) updateBuffer() {
	prompt := []byte(uiStartPrompt)

	promptEntered := len(world.ConnectPromptText) != 0
	if promptEntered {
		prompt = append(prompt, []byte("\n\n"+world.ConnectPromptText)...)
		prompt = append(prompt, []byte("\n\n"+uiConnectPrompt)...)
		prompt = append(prompt, []byte("\n\n"+uiRemotePrompt)...)
	} else {
		prompt = append(prompt, []byte("\n\n"+uiComputerPrompt)...)
		prompt = append(prompt, []byte("\n\n"+uiHostPrompt)...)
		prompt = append(prompt, []byte("\n\n"+uiRemotePrompt)...)
	}

	u.buffer.Clear()
	u.buffer.Write(prompt)
}

func (u *UISystem) Update(e gohan.Entity) error {
	if !u.initialized {
		u.initialize()
	}

	if !world.ConnectPromptVisible {
		return nil
	}

	var a string
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit0) {
		a += "0"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit1) {
		a += "1"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit2) {
		a += "2"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit3) {
		a += "3"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit4) {
		a += "4"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit5) {
		a += "5"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit6) {
		a += "6"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit7) {
		a += "7"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit8) {
		a += "8"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyDigit9) {
		a += "9"
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyPeriod) {
		a += "."
	}
	if inpututil.IsKeyJustPressed(ebiten.KeySemicolon) && ebiten.IsKeyPressed(ebiten.KeyShift) {
		a += ":"
	}
	if a != "" {
		world.ConnectPromptText += a
		u.updateBuffer()
	}
	if inpututil.IsKeyJustPressed(ebiten.KeyBackspace) && len(world.ConnectPromptText) != 0 {
		world.ConnectPromptText = world.ConnectPromptText[:len(world.ConnectPromptText)-1]
		u.updateBuffer()
	}

	if inpututil.IsKeyJustPressed(ebiten.KeyEnter) || inpututil.IsKeyJustPressed(ebiten.KeyKPEnter) {
		log.Println("Start game")
		return nil
	}

	return etk.Update()
}

func (u *UISystem) Draw(e gohan.Entity, screen *ebiten.Image) error {
	if !u.initialized {
		u.initialize()
	}

	if !world.ConnectPromptVisible {
		return nil
	}

	err := etk.Draw(screen)
	if err != nil {
		return err
	}

	if world.Debug != 0 {
		u.debugImg.Clear()
		ebitenutil.DebugPrintAt(u.debugImg, fmt.Sprintf("ENT %d\nUPD %d\nDRA %d\nTPS %0.0f\nFPS %0.0f", gohan.CurrentEntities(), gohan.CurrentUpdates(), gohan.CurrentDraws(), ebiten.ActualTPS(), ebiten.ActualFPS()), 2, 0)
		op := &ebiten.DrawImageOptions{}
		op.GeoM.Scale(2, 2)
		screen.DrawImage(u.debugImg, op)
	}
	return nil
}
